@@ -7,8 +7,8 @@
 
 sampleN.scABEL <- function(alpha=0.05, targetpower=0.8, theta0, theta1, 
                            theta2, CV, design=c("2x3x3", "2x2x4"), 
-                           regulator=c("EMA", "FDA"), nsims=1E6,
-                           nstart, print=TRUE, details=TRUE)
+                           regulator=c("EMA", "FDA"), nsims=1E5,
+                           nstart, print=TRUE, details=TRUE, setseed=TRUE)
 {
   if (missing(theta1) & missing(theta2)) theta1 <- 0.8
   if (missing(theta0)) theta0 <- 0.95
@@ -64,12 +64,12 @@ sampleN.scABEL <- function(alpha=0.05, targetpower=0.8, theta0, theta1,
   # for later enhancement taking into account the 
   # subject-by-formulation interaction
   sD2  <- 0  
-  sWT2 <- log(1.0 + CV[1]^2)
-  if (length(CV)==2) sWR2 <- log(1.0 + CV[2]^2) else sWR2 <-sWT2
-  CVwR <- sqrt(exp(sWR2)-1)
-  CVwT <- sqrt(exp(sWT2)-1)
-  # sd^2 of the differences T-R from their components
-  sd2  <- (sD2 + (sWT2 + sWR2)/2) # is this correct for partial replicate?
+  CVwT <- CV[1]
+  if (length(CV)==2) CVwR <- CV[2] else CVwR <- CVwT
+  s2WT <- log(1.0 + CVwT^2)
+  s2WR <- log(1.0 + CVwR^2)
+  # sd^2 (variance) of the differences T-R from their components
+  sd2  <- (sD2 + (s2WT + s2WR)/2) # is this correct for partial replicate?
   mlog <- log(theta0)
   
   if (print){
@@ -77,12 +77,13 @@ sampleN.scABEL <- function(alpha=0.05, targetpower=0.8, theta0, theta1,
     cat("            Sample size estimation\n")
     cat("---------------------------------------------\n")
     cat("Study design: ",design,"\n")
-    cat("log-transformed data (multiplicative model)\n\n")
-    cat("alpha = ",alpha,", target power = ", targetpower,"\n", sep="")
+    cat("log-transformed data (multiplicative model)\n")
+    cat(nsims,"studies simulated.\n\n")
+    cat("alpha  = ",alpha,", target power = ", targetpower,"\n", sep="")
     cat("CVw(T) = ",CVwT,"; CVw(R) = ",CVwR,"\n", sep="")
     cat("Null (true) ratio = ",theta0,"\n", sep="")
-    cat("PE constraints    =",theta1,"...", theta2,"\n")
-    cat("Regulatory body:",regulator,"\n")
+    cat("ABE limits / PE constraints =",theta1,"...", theta2,"\n")
+    cat("Regulatory settings:",regulator,"\n")
     if (details) { 
       cat("- CVswitch = ", CVswitch)
       if (is.finite(CVcap)){
@@ -97,7 +98,7 @@ sampleN.scABEL <- function(alpha=0.05, targetpower=0.8, theta0, theta1,
   # -----------------------------------------------------------------
   # nstart? from sampleN0 with widened limits
   # does'nt fit really good if theta0>=1.2! ways out?
-  ltheta1 <- -sqrt(sWR2)*r_const
+  ltheta1 <- -sqrt(s2WR)*r_const
   ltheta2 <- -ltheta1
   if (CVwR <= CVswitch){
     ltheta1 <- log(theta1)
@@ -110,10 +111,12 @@ sampleN.scABEL <- function(alpha=0.05, targetpower=0.8, theta0, theta1,
   if (missing(nstart)){
     n <- .sampleN0(alpha=alpha, targetpower, ltheta1, ltheta2, diffm=mlog, 
                    se=sqrt(sd2), steps=seqs, bk=bk)
-  } else n <- nstart           
-  # next is an empirical observation for the EMA settings
-  # both together give n=2*n in case of CVwR>0.5
-  # TODO: check this if CVwR != CVwT
+  } else n <- seqs*round(nstart/seqs)           
+# empirical corrections not released because they are not unique
+# for all settings
+# next is an empirical observation for the EMA settings
+# both together give n=2*n in case of CVwR>0.5
+# TODO: check this if CVwR != CVwT
 #  if (targetpower<0.9){
 #    if ((theta0>=1.2 | theta0<=0.833) & CVwR>=0.45) n <- 1.5*n
 #    if ((theta0>=1.2 | theta0<=0.833) & CVwR>=0.5)  n <- 1.3333*n
@@ -129,15 +132,15 @@ sampleN.scABEL <- function(alpha=0.05, targetpower=0.8, theta0, theta1,
   sdm  <- sqrt(sd2*fact)
   df   <- eval(dfe)
   dfRR <- eval(dfRRe)
-  p <- .power.scABEL(mlog, sdm, fact, sd2, df, sWR2, dfRR, nsims, 
+  
+  if(setseed) set.seed(123456)
+  p <- .power.scABEL(mlog, sdm, fact, sd2, df, s2WR, dfRR, nsims, 
                      ln_lBEL=log(theta1),ln_uBEL=log(theta2), 
                      CVswitch, r_const, CVcap, alpha=alpha)
   pwr <- as.numeric(p["BE"]);
   
   if (details) {
     cat("\nSample size search\n")
-    # parallel group design is now in terms of ntotal
-    #if (d.no == 0) cat("(n is sample size per group)\n") #parallel group design
     cat(" n     power\n")
     # do not print first too high
     # this is for cases with only one step-down and than step up
@@ -146,9 +149,6 @@ sampleN.scABEL <- function(alpha=0.05, targetpower=0.8, theta0, theta1,
   iter <- 0; imax=100
   nmin <- 6
   # iter>100 is emergency brake
-  # this is eventually not necessary, depends on quality of sampleN0
-  # in experimentation I have seen max of six steps
-  # reformulation with only one loop does not shorten the code considerable
   # --- loop until power <= target power, step-down
   while (pwr>targetpower) {
     if (n<=nmin) { 
@@ -156,7 +156,8 @@ sampleN.scABEL <- function(alpha=0.05, targetpower=0.8, theta0, theta1,
       break
     }
     n  <- n-seqs     # step down if start power is to high
-#    does not function stepsize too big in some cases    
+    iter <- iter + 1
+# rising the stepsize does not function stepsize too big in some cases    
 #    if (abs(pwr-targetpower)>0.03) n  <- n-seqs
     fact <- bk/n
     # sd of the sample mean T-R (point estimator)
@@ -164,19 +165,19 @@ sampleN.scABEL <- function(alpha=0.05, targetpower=0.8, theta0, theta1,
     df   <- eval(dfe)
     dfRR <- eval(dfRRe)
     
-    p <- .power.scABEL(mlog, sdm, fact, sd2, df, sWR2, dfRR, nsims, 
+    if(setseed) set.seed(123456)
+    p <- .power.scABEL(mlog, sdm, fact, sd2, df, s2WR, dfRR, nsims, 
                        ln_lBEL=log(theta1),ln_uBEL=log(theta2), 
                        CVswitch, r_const, CVcap, alpha=alpha)
     pwr <- as.numeric(p["BE"]);
     
-    # do not print first step down
-    if (details) cat( n," ", formatC(pwr, digits=6),"\n")
+    if (details) cat( n," ", formatC(pwr, digits=6, format="f"),"\n")
     if (iter>imax) break  
     # loop results in n with power too low
     # must step one up again. is done in the next loop
   }
   while (pwr<targetpower) {
-    n    <- n+seqs
+    n    <- n+seqs   # step-up
 #    does not function stepsize too big in some cases    
 #    if (abs(pwr-targetpower)>0.03) n  <- n+seqs
     iter <- iter+1
@@ -185,7 +186,8 @@ sampleN.scABEL <- function(alpha=0.05, targetpower=0.8, theta0, theta1,
     df   <- eval(dfe)
     dfRR <- eval(dfRRe)
     
-    p <- .power.scABEL(mlog, sdm, fact, sd2, df, sWR2, dfRR, nsims, 
+    if(setseed) set.seed(123456)
+    p <- .power.scABEL(mlog, sdm, fact, sd2, df, s2WR, dfRR, nsims, 
                        ln_lBEL=log(theta1),ln_uBEL=log(theta2), 
                        CVswitch, r_const, CVcap, alpha=alpha)
     pwr <- as.numeric(p["BE"]);
